<?php

if ( !defined( 'ABSPATH' ) ) {
    exit;
}


if ( !class_exists( 'WOOPCD_PartialCOD_Order_Activity_Cron' ) ) {

    class WOOPCD_PartialCOD_Order_Activity_Cron {

        private $hash_modified;
        private $hash_store;
        private $run_scheduler_hook;
        private $run_batch_hook;

        public function __construct() {
            // initialize cron params
            $this->hash_modified = false;
            $this->hash_store = array();
            $this->run_scheduler_hook = 'partialcod-cron/run-scheduler';
            $this->run_batch_hook = 'woopcd_partialcod/run_batch';

            $next_check = get_transient( 'wp_partialcod_cron_scheduler_next' );
            //check if the schedulers needs any attensionx
            if ( !($next_check) || $next_check <= current_time( 'timestamp' ) ) {

                if ( $this->has_active_schedulers() ) {
                    add_action( $this->run_scheduler_hook, array( $this, 'run_scheduler' ), 10, 1 );
                    add_action( $this->run_batch_hook, array( $this, 'run_batch' ), 10, 1 );
                }

                $this->init_crons();
            } else if ( get_transient( 'wp_partialcod_cron_scheduler_active' ) ) {
                add_action( $this->run_scheduler_hook, array( $this, 'run_scheduler' ), 10, 1 );
                add_action( $this->run_batch_hook, array( $this, 'run_batch' ), 10, 1 );
            }
        }

        public function init_crons() {
            //try activate/re-activate schedulers
            foreach ( $this->get_schedulers() as $settings ) {
                $this->maybe_deactivate( $settings );
                $this->maybe_activate( $settings );
            }
            //clean scheduler database
            $this->clean_crons();
            //save scheduler hash values
            $this->save_hash_data();

            //set when schedulers will need any attensions
            $next_check = current_time( 'timestamp' ) + (DAY_IN_SECONDS * 7);

            set_transient( 'wp_partialcod_cron_scheduler_next', $next_check, $next_check );
            if ( $this->has_active_schedulers() ) {
                set_transient( 'wp_partialcod_cron_scheduler_active', true, $next_check );
            }
        }

        public function run_scheduler( $option_id ) {
            //get order ids from orders database and run order batchs
            $settings = $this->get_scheduler_by_id( $option_id );
            if ( !count( $settings ) ) {
                return;
            }
            // get order_ids from database
            $db_order_ids = $this->query_db( $settings );

            //run batchs or triggers based on number of order_ids and batch size
            if ( !is_numeric( $settings[ 'batch_size' ] ) ) {
                $this->run_trigger( $db_order_ids, $settings[ 'option_id' ] );
            } else if ( $db_order_ids > $settings[ 'batch_size' ] ) {
                if ( !is_numeric( $settings[ 'batch_intervals' ] ) ) {
                    $settings[ 'batch_intervals' ] = 0;
                }
                //create batch params
                $batch_data = array(
                    'order_ids' => $db_order_ids,
                    'count' => count( $db_order_ids ),
                    'batch_size' => $settings[ 'batch_size' ],
                    'batch_intervals' => $settings[ 'batch_intervals' ],
                    'batch_point' => 0
                );
                //update batch params
                update_option( 'woopcd_partialcod_batch_' . $settings[ 'option_id' ], $batch_data );
                // run first batch
                $this->run_batch( $settings[ 'option_id' ] );
            } else {
                //run triggers
                $this->run_trigger( $db_order_ids, $settings[ 'option_id' ] );
            }
        }

        public function run_batch( $option_id ) {
            
            // get batch order ids
            $option_key = 'woopcd_partialcod_batch_' . $option_id;

            $batch_data = get_option( $option_key, array() );

            if ( !count( $batch_data ) ) {
                return;
            }

            //get batch position
            $batch_point = $batch_data[ 'batch_point' ];

            // get batch order ids
            $order_ids = array();
            for ( $i = $batch_point; $i < $batch_data[ 'count' ]; $i++ ) {

                $order_ids[] = $batch_data[ 'order_ids' ][ $i ];
                $batch_point++;
                if ( count( $order_ids ) >= $batch_data[ 'batch_size' ] ) {
                    break;
                }
            }

            // run triggers based batch order ids
            if ( count( $order_ids ) ) {
                $this->run_trigger( $order_ids, $option_id );
            }


            // clean database if it's the last batch
            if ( $batch_point >= ($batch_data[ 'count' ] - 1) ) {
                delete_option( $option_key );
                return;
            }


            // create single event based on next batch
            if ( $batch_point < ($batch_data[ 'count' ] - 1) ) {
                $batch_data[ 'batch_point' ] = $batch_point;
                update_option( $option_key, $batch_data );

                $next_batch_event = wp_get_scheduled_event( $this->run_batch_hook, array( $option_id ) );
                if ( !$next_batch_event ) {
                    wp_schedule_single_event( current_time( 'timestamp' ) + $batch_data[ 'batch_intervals' ], $this->run_batch_hook, array( $option_id ) );
                }
            }
        }

        private function run_trigger( $order_ids, $scheduler_id ) {
            if ( !$this->is_triggers_active() ) {
                return;
            }
            // trigger order autopilots based on each order ids and options_id (as $scheduler_id)
            foreach ( $order_ids as $order_id ) {
                if ( $order_id > 0 ) {
                    do_action( 'woopcd_partialcod/cron-trigger', $order_id, $scheduler_id );
                }
            }
        }

        private function clean_crons() {
            // get hash values to be removed
            $all_ids = $this->get_scheduler_ids();
            $all_hash = $this->get_all_hash();
            // remove hash values for deactivated schedulers to clean the database
            foreach ( array_keys( $all_hash ) as $option_id ) {
                if ( !in_array( $option_id, $all_ids ) ) {
                    $this->remove_hash( $option_id );
                }
            }
        }

        private function maybe_activate( $settings ) {
            //activate/re-activate the schedule
            if ( $this->is_active( $settings ) ) {
                $this->activate( $settings );
            }
        }

        private function maybe_deactivate( $settings ) {
            // try deactivate if needed (mostly fro re-activation)
            if ( !$this->is_active( $settings ) ) {
                $this->deactivate( $settings[ 'option_id' ] );
            }
            // generate/re-generate hash value based on params
            $hash = $this->generate_hash( $settings );
            $stored_hash = $this->get_hash( $settings[ 'option_id' ] );
            // save the newly generated hash and deactivate the event
            if ( $hash != $stored_hash ) {
                $this->deactivate( $settings[ 'option_id' ] );
                $this->change_hash( $settings );
            }
        }

        private function activate( $settings ) {

            // activate schedules if it's not active
            $option_id = $settings[ 'option_id' ];
            $next_event_interval = wp_get_scheduled_event( $this->run_scheduler_hook, array( $option_id ) );

            if ( !$next_event_interval ) {

                //get schedule datetime
                $current_time = current_time( 'timestamp' );

                $recurrence = $settings[ 'intervals' ];
                $start_from = $current_time;
                // get schedule datetime if specified
                $start_from_str = '';
                if ( '' != $settings[ 'start_date' ] ) {
                    $start_from_str = $settings[ 'start_date' ];
                }
                if ( '' != $start_from_str && '' != $settings[ 'start_time' ] ) {
                    $start_from_str = $start_from_str . ' ' . $settings[ 'start_time' ];
                }

                if ( '' != $start_from_str ) {
                    $start_from = $this->get_datetime_int( $start_from_str, '' == $settings[ 'start_time' ] );
                }

                if ( $current_time > $start_from ) {
                    $start_from = $current_time;
                }
                // create the schedule
                wp_schedule_event( $start_from, $recurrence, $this->run_scheduler_hook, array( $option_id ) );
            }
        }

        private function deactivate( $option_id ) {
            //deactivate scheduler if it's still active
            $next_event_interval = wp_get_scheduled_event( $this->run_scheduler_hook, array( $option_id ) );

            if ( $next_event_interval ) {
                wp_unschedule_event( $next_event_interval->timestamp, $this->run_scheduler_hook, array( $option_id ) );
            }



            //deactivate batch if it's still active
            $next_batch_event = wp_get_scheduled_event( $this->run_batch_hook, array( $option_id ) );

            if ( $next_batch_event ) {
                $option_key = 'woopcd_partialcod_batch_' . $option_id;
                delete_option( $option_key );
                wp_unschedule_event( $next_batch_event->timestamp, $this->run_batch_hook, array( $option_id ) );
            }
        }

        private function change_hash( $settings ) {
            // re-generate hash key based on new params
            $this->hash_modified = true;
            $hash = $this->generate_hash( $settings );
            $this->hash_store[ $settings[ 'option_id' ] ] = $hash;
        }

        private function remove_hash( $option_id ) {
            // remove hash from store
            if ( $this->hash_store[ $option_id ] ) {
                $this->hash_modified = true;
                unset( $this->hash_store[ $option_id ] );
            }
        }

        private function save_hash_data() {
            // store hash keys for persistence schedules management
            if ( $this->hash_modified ) {
                update_option( 'woopcd_partialcod_cron_hash', $this->hash_store, true );
            }
        }

        private function get_hash( $option_id ) {
            //get stored hash by option_id
            $stored_h = $this->get_all_hash();

            if ( isset( $stored_h[ $option_id ] ) ) {
                return $stored_h[ $option_id ];
            }

            return '';
        }

        private function get_all_hash() {
            // get stored hash keys for persistence schedules management
            if ( !count( $this->hash_store ) ) {
                $this->hash_store = get_option( 'woopcd_partialcod_cron_hash', array() );
            }

            return $this->hash_store;
        }

        private function generate_hash( $settings ) {
            // creates scheduler hash key based on scheduler params
            $hash_obj = array(
                'option_id' => $settings[ 'option_id' ],
                'intervals' => $settings[ 'intervals' ],
                'start_date' => $settings[ 'start_date' ],
                'start_time' => $settings[ 'start_time' ],
            );

            return md5( wp_json_encode( $hash_obj, 0 ) );
        }

        private function has_active_schedulers() {
            // check for any active schedulers
            $active_schedulers_count = 0;
            foreach ( $this->get_schedulers() as $settings ) {
                if ( $this->is_active( $settings ) ) {
                    $active_schedulers_count++;
                }
            }
            return ($active_schedulers_count > 0);
        }

        private function is_active( $settings ) {
            return ( '' != $settings[ 'intervals' ] );
        }

        private function get_schedulers() {
            // get scheduler from woopcd_partialcod option settings
            $cron_sechdulers = array();
            foreach ( WOOPCD_PartialCOD::get_option( 'order_cron_sechduler', array() ) as $cron_sechduler ) {
                $cron_sechdulers[ $cron_sechduler[ 'option_id' ] ] = $cron_sechduler;
            }

            return $cron_sechdulers;
        }

        private function get_scheduler_by_id( $option_id ) {
            // get single scheduler by option_id
            $cron_sechdulers = $this->get_schedulers();
            if ( isset( $cron_sechdulers[ $option_id ] ) ) {
                return $cron_sechdulers[ $option_id ];
            }
            return array();
        }

        private function get_scheduler_ids() {
            //get scheduler ids from schedulers
            $cron_sechdulers = $this->get_schedulers();
            if ( count( $cron_sechdulers ) ) {
                return array_keys( $cron_sechdulers );
            }
            return array();
        }

        private function is_triggers_active() {
            return WOOPCD_PartialCOD_Order_Activity_Trigger_Types::get_trigger_type_is_active( 'cron_schedulers' );
        }

        private function query_db( $settings ) {
            global $wpdb;

            $from_date = '';
            // prepare query
            if ( 'timespan' == $settings[ 'query_from_mode' ] ) {
                $from_date = $this->get_datetime_from_span( $settings[ 'query_timespan' ], $settings[ 'query_timespan_mode' ] );
            }
            if ( 'date' == $settings[ 'query_from_mode' ] && '' != $settings[ 'query_date' ] ) {
                $from_date = $settings[ 'query_date' ] . ' 00:00:00';
            }

            $sql = "SELECT posts.id FROM $wpdb->posts as posts";

            if ( is_array( $settings[ 'query_payment_methods' ] ) ) {
                $sql = $sql . " LEFT JOIN {$wpdb->postmeta} AS mt_pay ON posts.ID = mt_pay.post_id";
            }

            if ( '' != $settings[ 'query_custom_meta_key' ] ) {
                $sql = $sql . " LEFT JOIN {$wpdb->postmeta} AS mt_custom ON posts.ID = mt_custom.post_id";
            }


            $sql = $sql . " WHERE (posts.post_type='shop_order')";


            if ( '' != $from_date ) {
                $sql = $sql . $wpdb->prepare( " AND (posts.post_date>='" . $from_date . "')" );
            }

            if ( is_array( $settings[ 'query_status' ] ) ) {
                $order_statuses = "'" . implode( "', '", $settings[ 'query_status' ] ) . "'";
                $sql = $sql . $wpdb->prepare( " AND (posts.post_status IN(" . $order_statuses . "))" );
            }

            if ( is_array( $settings[ 'query_payment_methods' ] ) ) {
                $payment_methods = "'" . implode( "', '", $settings[ 'query_payment_methods' ] ) . "'";
                $sql = $sql . $wpdb->prepare( " AND (mt_pay.meta_key='_payment_method')" );
                $sql = $sql . $wpdb->prepare( " AND (mt_pay.meta_value IN (" . $payment_methods . "))" );
            }


            if ( '' != $settings[ 'query_custom_meta_key' ] ) {
                $sql = $sql . $wpdb->prepare( " AND (mt_custom.meta_key = '" . $settings[ 'query_custom_meta_key' ] . "')" );
                $sql = $sql . $wpdb->prepare( " AND (mt_custom.meta_value = '" . $settings[ 'query_custom_meta_value' ] . "')" );
            }

            $sql = $sql . $wpdb->prepare( " ORDER BY posts.post_date " . $settings[ 'query_sort' ] );

            $sql = $sql . $wpdb->prepare( " LIMIT " . $settings[ 'query_limit' ] );

            // query database
            $db_results = $wpdb->get_results( $sql, ARRAY_A );

            // get order ids from database query result
            $results = array();
            if ( count( $db_results ) ) {
                foreach ( $db_results as $db_result ) {
                    $results[] = ( int ) $db_result[ 'id' ];
                }
            }


            return $results;
        }

        private function get_datetime_int( $datetime, $add_time = false ) {

            // get datetime integer based on human readable datetime
            $datetime_format = 'Y-m-d H:i:s';

            if ( $add_time ) {
                $datetime = $datetime . ' 00:00:00';
            }

            $datetime_obj = DateTime::createFromFormat( $datetime_format, $datetime );


            return $datetime_obj->format( 'U' );
        }

        private function get_datetime_from_span( $timespan, $timespan_mode ) {
            // get current time as time-span if timespan is not numeric
            if ( !is_numeric( $timespan ) ) {
                $datetime = DateTime::createFromFormat( 'U', current_time( 'timestamp' ) );
                return $datetime->format( 'Y-m-d' ) . ' ';
            }

            // get time-span based on time-span mode
            $seconds = 0;
            switch ( $timespan_mode ) {
                case 'minutes':
                    $seconds = MINUTE_IN_SECONDS * $timespan;
                    break;
                case 'hours':
                    $seconds = HOUR_IN_SECONDS * $timespan;
                    break;
                case 'days':
                    $seconds = DAY_IN_SECONDS * $timespan;
                    break;
                case 'weeks':
                    $seconds = WEEK_IN_SECONDS * $timespan;
                    break;
                case 'months':
                    $seconds = MONTH_IN_SECONDS * $timespan;
                    break;
                case 'years':
                    $seconds = YEAR_IN_SECONDS * $timespan;
                    break;
                default:
                    $seconds = MINUTE_IN_SECONDS * $timespan;
                    break;
            }

            $datetime = DateTime::createFromFormat( 'U', current_time( 'timestamp' ) - $seconds );
            return $datetime->format( 'Y-m-d H:i:s' );
        }

    }

    new WOOPCD_PartialCOD_Order_Activity_Cron();
}