<?php

if ( !defined( 'ABSPATH' ) ) {
    exit;
}


if ( !class_exists( 'WOOPCD_PartialCOD_Order_Activity_Engine' ) ) {

    class WOOPCD_PartialCOD_Order_Activity_Engine {

        private $order_activity_list;

        public function __construct() {
            $this->order_activity_list = array();
        }

        public function process_order_data( $order_data ) {

            $source = $order_data[ 'wc' ][ 'source' ];

            if ( isset( $this->order_activity_list[ $source[ 'trigger' ] ] ) ) {
                $order_data[ 'activities' ] = $this->order_activity_list[ $source[ 'trigger' ] ];
                return $order_data;
            }

            $order_data = $this->get_activities( $order_data, $source );

            if ( isset( $order_data[ 'activities' ] ) ) {
                $this->order_activity_list[ $source[ 'trigger' ] ] = $order_data[ 'activities' ];
            }

            return $order_data;
        }

        private function get_activities( $order_data, $source ) {

            $method_id = $this->get_method_id( $order_data );
            $rules_settings = $this->get_rules_settings( $method_id );

            //check if activities rules should apply
            if ( 'no' == $rules_settings[ 'mode' ] ) {
                return $order_data;
            }

            // go through the rules and apply them
            foreach ( $this->get_rule_options_by_trigger_type( $source, $method_id ) as $rule ) {

                $rule_head = array(
                    'apply_mode' => $rule[ 'apply_mode' ],
                    'enable' => $rule[ 'enable' ],
                    'rule_id' => $rule[ 'rule_id' ],
                    'settings' => $rules_settings,
                    'method_id' => $method_id,
                    'module' => 'order-activities',
                );


                $rule_conditions = $rule[ 'rule_conditions' ];
                $rule_conditions[ 'method_id' ] = $method_id;
                $rule_conditions[ 'module' ] = 'order-activities';

                if ( !$this->can_apply_rule( $rule_head, $rule_conditions, $order_data ) ) {
                    continue;
                }


                $activities_args = $this->get_activities_args_from_rule( $method_id, $rule );

                $activity_list = WOOPCD_PartialCOD_Activity_Types::get_activities( array(), $activities_args, $order_data );

                $activities = array(
                    'delay_activities' => $rule[ 'delay_activities' ],
                );

                if ( 'yes' == $rule[ 'delay_activities' ] ) {

                    $activities[ 'timer_mode' ] = $rule[ 'timer_mode' ];
                    $activities[ 'timer' ] = is_numeric( $rule[ 'timer' ] ) ? $rule[ 'timer' ] : 0;
                }

                $activities[ 'list' ] = $activity_list;


                if ( has_filter( 'woopcd_partialcod/order-activities/gotten-activity-list' ) ) {
                    $activity_list = apply_filters( 'woopcd_partialcod/order-activities/gotten-activities-list', $activity_list, $rule_head[ 'rule_id' ], $order_data );
                }


                $order_data = $this->add_rule_options( $order_data, $activities, $rule_head );
            }

            return $order_data;
        }

        private function get_activities_args_from_rule( $method_id, $rule ) {

            $activities_args = array();

            if ( isset( $rule[ 'activities' ] ) ) {
                $activities_args = $rule[ 'activities' ];
            }

            $arg_keys = array_keys( $activities_args );

            foreach ( $arg_keys as $key ) {
                $activities_args[ $key ][ 'module' ] = 'order-activities';
                $activities_args[ $key ][ 'rule_id' ] = $rule[ 'rule_id' ];
                $activities_args[ $key ][ 'method_id' ] = $method_id;
            }

            return $activities_args;
        }

        private function can_apply_rule( $rule_head, $rule_conditions, $order_data ) {


            // allows active rules only
            if ( 'no' == $rule_head[ 'enable' ] ) {
                return false;
            }

            $found_activities = isset( $order_data[ 'activities' ] );

            if ( 'if_others' == $rule_head[ 'apply_mode' ] && !$found_activities ) {
                return false;
            }

            if ( 'if_no_others' == $rule_head[ 'apply_mode' ] && $found_activities ) {
                return false;
            }

            if ( 'first' == $rule_head[ 'settings' ][ 'mode' ] && $found_activities ) {
                return false;
            }


            $bool_val = true;

            // allows other plugins to check if the rule should apply
            if ( has_filter( 'woopcd_partialcod/order-activities/can-apply-rule' ) ) {
                $bool_val = apply_filters( 'woopcd_partialcod/order-activities/can-apply-rule', $bool_val, $rule_head, $order_data );
            }

            // there no need for validations
            if ( false == $bool_val ) {
                return $bool_val;
            }

            // validate rule

            $bool_val = WOOPCD_PartialCOD_Order_Condition_Types::validate_rule_conditions( $rule_conditions, $order_data );


            return $bool_val;
        }

        private function add_rule_options( $order_data, $rule_activities, $rule_head ) {

            $s_mode = $rule_head[ 'settings' ][ 'mode' ];

            //remove prev activities (if last valid rules or only rule)
            if ( 'last' == $s_mode || 'only_this' == $rule_head[ 'apply_mode' ] ) {
                $order_data[ 'activities' ] = array();
            }

            $order_data[ 'activities' ][ $rule_head[ 'rule_id' ] ] = $rule_activities;

            // allows other plugins to modify options data once added
            if ( has_filter( 'woopcd_partialcod/order-activities/added-rule-activities' ) ) {
                $order_data[ 'activities' ][ $rule_head[ 'rule_id' ] ] = apply_filters( 'woopcd_partialcod/order-activities/added-rule-activities', $order_data[ 'activities' ][ $rule_head[ 'rule_id' ] ], $rule_head );
            }

            return $order_data;
        }

        private function get_rule_options_by_trigger_type( $source, $method_id ) {

            $rules = array();

            foreach ( $this->get_rule_options( $method_id ) as $rule ) {

                if ( $this->get_can_trigger_activities( $source, $rule ) ) {

                    $rules[] = $rule;
                }
            }

            return $rules;
        }

        private function get_can_trigger_activities( $source, $rule ) {

            if ( isset( $rule[ 'activities_trigger' ] ) && $source[ 'trigger' ] == $rule[ 'activities_trigger' ] ) {

                if ( has_filter( 'woopcd_partialcod/order-activities/can-trigger-' . $source[ 'trigger' ] . '-activities' ) ) {

                    return apply_filters( 'woopcd_partialcod/order-activities/can-trigger-' . $source[ 'trigger' ] . '-activities', $source, $rule );
                }

                return true;
            }

            return false;
        }

        private function get_rule_options( $method_id ) {

            $rules = array();
            foreach ( WOOPCD_PartialCOD::get_option( $method_id . '_order_activity_rules', array() ) as $rule ) {
                $rules[] = $rule;
            }
            return $rules;
        }

        private function get_rules_settings( $method_id ) {
            $default = array(
                'mode' => 'no',
            );
            return WOOPCD_PartialCOD::get_option( $method_id . '_order_activity_rules_settings', $default );
        }

        private function get_method_id( $order_data ) {

            if ( isset( $order_data[ 'wc' ][ 'risk_free' ][ 'method_id' ] ) ) {
                return $order_data[ 'wc' ][ 'risk_free' ][ 'method_id' ];
            }

            if ( isset( $order_data[ 'wc' ][ 'payment_method' ][ 'method_id' ] ) ) {
                return $order_data[ 'wc' ][ 'payment_method' ][ 'method_id' ];
            }
            return '';
        }

    }

}
