<?php

if ( !defined( 'ABSPATH' ) ) {
    exit;
}


if ( !class_exists( 'WOOPCD_PartialCOD_RiskFree_Engine' ) ) {

    class WOOPCD_PartialCOD_RiskFree_Engine {

        private $riskfree_details;

        public function __construct() {
            $this->riskfree_details = array();
        }

        public function process_cart_data( $cart_data ) {

            $source = $cart_data[ 'wc' ][ 'source' ];

            if ( 'gateways' != $source && 'riskfree_details' != $source ) {
                return $cart_data;
            }

            if ( isset( $this->riskfree_details[ $source ] ) ) {
                $cart_data[ 'riskfee' ] = $this->riskfree_details[ $source ];
                return $cart_data;
            }

            if ( isset( $cart_data[ 'riskfee' ] ) ) {
                unset( $cart_data[ 'riskfee' ] );
            }

            $cart_data = $this->get_riskfee( $cart_data );

            if ( isset( $cart_data[ 'riskfee' ] ) ) {
                $this->riskfree_details[ $source ] = $cart_data[ 'riskfee' ];
            }

            return $cart_data;
        }

        private function get_riskfee( $cart_data ) {

            $method_ids = WOOPCD_PartialCOD_Cart::get_method_ids();

            foreach ( $method_ids as $method_id ) {
                $cart_data = $this->get_method_riskfee( $cart_data, $method_id );
            }

            return $cart_data;
        }

        private function get_method_riskfee( $cart_data, $method_id ) {

            $rules_settings = $this->get_rules_settings( $method_id );

            //check if riskfree rules should apply
            if ( 'no' == $rules_settings[ 'mode' ] ) {
                return $cart_data;
            }


            // go through the rules and apply them
            foreach ( $this->get_rule_options( $method_id ) as $rule ) {

                // prevents applying multiple rules if method not selected
                if ( $this->is_added( $method_id, $cart_data ) && !$this->is_selected( $method_id, $cart_data ) ) {
                    return $cart_data;
                }



                // validate and apply the rule
                $rule_head = array(
                    'enable' => $rule[ 'enable' ],
                    'rule_id' => $rule[ 'rule_id' ],
                    'settings' => $rules_settings,
                    'method_id' => $method_id,
                    'module' => 'partial-payment',
                );

                $rule_conditions = $rule[ 'rule_conditions' ];
                $rule_conditions[ 'method_id' ] = $method_id;
                $rule_conditions[ 'module' ] = 'partial-payment';


                if ( !$this->can_apply_rule( $rule_head, $rule_conditions, $cart_data ) ) {
                    continue;
                }


                $is_selected = $this->is_selected( $method_id, $cart_data );
                $rule_id = $rule[ 'rule_id' ];


                $risk_free = array(
                    $rule_id => array(
                        'is_selected' => $is_selected,
                    ),
                );

                if ( true == $is_selected ) {

                    if ( isset( $rule[ 'allow_methods' ] ) ) {
                        $risk_free[ $rule_id ][ 'allow_methods' ] = $rule[ 'allow_methods' ];
                    } else {
                        $risk_free[ $rule_id ][ 'allow_methods' ] = array();
                    }
                    $risk_free[ $rule_id ][ 'desc' ] = $rule[ 'desc' ];

                    $amounts_args = $this->get_amounts_args_from_rule( $rule, $rules_settings, $method_id );

                    $totals = $cart_data[ 'wc' ][ 'totals' ][ 'total' ];

                    $risk_free[ $rule_id ][ 'totals' ] = $totals;
                    $risk_free[ $rule_id ][ 'total_tax' ] = $cart_data[ 'wc' ][ 'totals' ][ 'total_tax' ];


                    if ( 0 < $totals ) {

                        $advance_pay = WOOPCD_PartialCOD_Payment_Amount_Types::calculate_amounts( 0, $amounts_args, $rules_settings, $cart_data );

                        if ( $advance_pay > $totals ) {
                            
                            $advance_pay = $totals;
                        }

                        $delivery_pay = $totals - $advance_pay;

                        $risk_free[ $rule_id ][ 'advance_pay' ] = $advance_pay;
                        $risk_free[ $rule_id ][ 'delivery_pay' ] = $delivery_pay;
                    } else {
                        
                        $risk_free[ $rule_id ][ 'advance_pay' ] = 0;
                        $risk_free[ $rule_id ][ 'delivery_pay' ] = 0;
                    }
                }


                if ( has_filter( 'woopcd_partialcod/partial-payment/gotten-options' ) ) {
                    
                    $risk_free = apply_filters( 'woopcd_partialcod/partial-payment/gotten-options', $risk_free, $rule_id, $cart_data );
                }



                $cart_data = $this->add_rule_riskfree( $cart_data, $risk_free, $rule_head );
            }

            return $cart_data;
        }

        private function get_amounts_args_from_rule( $rule, $method_id ) {
            $amounts_args = array();

            if ( isset( $rule[ 'payment_amounts' ] ) ) {
                $amounts_args = $rule[ 'payment_amounts' ];
            }

            $arg_keys = array_keys( $amounts_args );
            foreach ( $arg_keys as $key ) {
                $amounts_args[ $key ][ 'method_id' ] = $method_id;
                $amounts_args[ $key ][ 'module' ] = 'partial-payment';
                $amounts_args[ $key ][ 'rule_id' ] = $rule[ 'rule_id' ];
            }

            return $amounts_args;
        }

        private function can_apply_rule( $rule_head, $rule_conditions, $cart_data ) {

            if ( 'no' == $rule_head[ 'enable' ] ) {
                return false;
            }


            if ( 'first' == $rule_head[ 'settings' ][ 'mode' ] && $this->is_added( $rule_head[ 'method_id' ], $cart_data ) ) {
                return false;
            }

            $bool_val = true;

            // allows other plugins to check if the rule should apply
            if ( has_filter( 'woopcd_partialcod/partial-payment/can-apply-rule' ) ) {
                
                $bool_val = apply_filters( 'woopcd_partialcod/partial-payment/can-apply-rule', $bool_val, $rule_head, $cart_data );
            }

            // there no need for validations
            if ( false == $bool_val ) {
                return $bool_val;
            }

            // validate rule

            $bool_val = WOOPCD_PartialCOD_Condition_Types::validate_rule_conditions( $rule_conditions, $cart_data );


            return $bool_val;
        }

        private function add_rule_riskfree( $cart_data, $risk_free, $rule_head ) {
            $method_id = $rule_head[ 'method_id' ];
            $s_mode = $rule_head[ 'settings' ][ 'mode' ];



            // add option if it's the first option or method not selected
            if ( !$this->is_added( $method_id, $cart_data ) || !$this->is_selected( $method_id, $cart_data ) ) {
                $cart_data[ 'riskfee' ][ $method_id ] = $risk_free;
                return $cart_data;
            }



            if ( 'higher' == $s_mode ) {
                // add rules with higher fees only
                if ( $this->get_is_higher( $cart_data, $risk_free, $rule_head[ 'rule_id' ], $method_id ) ) {
                    $cart_data[ 'riskfee' ][ $method_id ] = $risk_free;
                }
            } else if ( 'lower' == $s_mode ) {
                // add rules with lower fees only
                if ( $this->get_is_lower( $cart_data, $risk_free, $rule_head[ 'rule_id' ], $method_id ) ) {
                    $cart_data[ 'riskfee' ][ $method_id ] = $risk_free;
                }
            } else {
                // add rule as usual
                $cart_data[ 'riskfee' ][ $method_id ] = $risk_free;
            }

            return $cart_data;
        }

        private function is_added( $method_id, $cart_data ) {
            return isset( $cart_data[ 'riskfee' ][ $method_id ] );
        }

        private function is_selected( $method_id, $cart_data ) {
            return (isset( $cart_data[ 'wc' ][ 'method_ids' ][ 'riskfree_id' ] ) && $method_id == $cart_data[ 'wc' ][ 'method_ids' ][ 'riskfree_id' ]);
        }

        private function get_is_higher( $cart_data, $risk_free, $rule_id, $method_id ) {
            $d_value = 0;

            if ( !isset( $cart_data[ 'riskfee' ][ $method_id ] ) ) {
                return true;
            }

            if ( isset( $cart_data[ 'riskfee' ][ $method_id ][ $rule_id ][ 'advance_pay' ] ) ) {
                $d_value = $cart_data[ 'riskfee' ][ $method_id ][ $rule_id ][ 'advance_pay' ];
            }

            return ($risk_free[ $rule_id ][ 'advance_pay' ] > $d_value);
        }

        private function get_is_lower( $cart_data, $risk_free, $rule_id, $method_id ) {
            $d_value = 0;

            if ( !isset( $cart_data[ 'riskfee' ][ $method_id ] ) ) {
                return true;
            }

            if ( isset( $cart_data[ 'riskfee' ][ $method_id ][ $rule_id ][ 'advance_pay' ] ) ) {
                $d_value = $cart_data[ 'riskfee' ][ $method_id ][ $rule_id ][ 'advance_pay' ];
            }

            return ($risk_free[ $rule_id ][ 'advance_pay' ] < $d_value);
        }

        private function get_rules_settings( $method_id ) {
            $default = array(
                'mode' => 'no',
            );
            return WOOPCD_PartialCOD::get_option( $method_id . '_riskfree_rules_settings', $default );
        }

        private function get_rule_options( $method_id ) {

            $rules = array();
            foreach ( WOOPCD_PartialCOD::get_option( $method_id . '_riskfree_rules', array() ) as $rule ) {
                $rules[] = $rule;
            }
            return $rules;
        }

    }

}